/*
 * mapping.c - routines for recording, displaying walked path.
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "cancan.h"
#include "term.h"

static char reverse_dir();

/*
 * mapping variables
 */
static char mappath[MAXMAPLEN];		/* circular path list */
static unsigned int mapstart;		/* index to first map entry */
static unsigned int mapend;		/* index one past last map entry */

#if (MAXMAPLEN - 1) & MAXMAPLEN
#define MAPINDEX(i) (((i) + MAXMAPLEN) % MAXMAPLEN)
#else
#define MAPINDEX(i) ((i) & (MAXMAPLEN - 1))
#endif

/*
 * return reverse direction
 */
static char reverse_dir(t)
char t;
{
    char *dirs = "denuwsden";		/* Reverend Johann von Denuwsden */
    char *p = strchr(dirs + 3, t);
    return p ? *(p - 3) : 0;
}

/*
 * remove steps either from map end or beginning.
 */
void remove_steps(steps)
int steps;
{
    int mapsize = MAPINDEX(mapend - mapstart);
    if(steps < 0) {
	mapend = mapsize + steps > 0 ? MAPINDEX(mapend + steps) : mapstart;
    } else if(mapsize > steps) {
	mapstart = MAPINDEX(mapend - steps);
    }
}

/*
 * retrace steps on map, and walk them back
 */
void retrace_steps(steps)
int steps;
{
    static char wcmd[2] = { ' ', '\n' };
    int mapsize = MAPINDEX(mapend - mapstart);
    if(steps <= 0) {
	if((steps += mapsize) < 0) return;
    } else if(steps > mapsize)
	steps = mapsize;
    while(steps) {
	steps--;
	mapend = MAPINDEX(mapend - 1);
	wcmd[0] = reverse_dir(mappath[mapend]);
	send_to_host(wcmd, 2);
	expect_prompt++;
    }
}

/*
 * show automatic map (latest steps) of the form s2esw14n
 */
void show_automap()
{
    unsigned int i = mapstart;
    if(i == mapend) {
	printf("#map empty\n");
    } else {
	/* NOTE: should we print map path using to_pager() ? */
	printf("#map: ");
	do {
	    char lastdir = mappath[i];
	    int count = 0;
	    do {
		count++;  i++;  i = MAPINDEX(i);
	    } while(i != mapend && mappath[i] == lastdir);
	    if(count > 1)
		printf("%d", count);
	    putc(lastdir, stdout);
	} while(i != mapend);
	newline();
    }
}

/*
 * add direction to automap
 */
void add_dir_to_map(dir)
char dir;
{
    if(!OPTION(OPT_MAZEMAPPING) && mapend != mapstart &&
	    dir == reverse_dir(mappath[MAPINDEX(mapend - 1)])) {
	mapend = MAPINDEX(mapend - 1);	/* retrace one step */
    } else {
	mappath[mapend] = dir;
	mapend = MAPINDEX(mapend + 1);
	if(mapend == mapstart)
	    mapstart = MAPINDEX(mapstart + 1);
    }
}
