/*
 * term.h
 */

extern int cols, lines;		/* terminal window size */
extern int cols_1, lines_1;	/* edit line width, last line */
extern int line0, col0;		/* origin of input line */
extern int metakey;
extern char clreoln[], clreoscr[], begoln[], modenorm[];
extern char key_left[], key_right[], key_up[], key_down[];

extern int edattr;		/* edit line attributes code */
extern char edattrbeg[], edattrend[];	/* and on/off sequences */

void set_terminal();
void reset_terminal();
void set_spec_keys();
void settermcap();
void keypad_bindings();
void default_bindings();
void moveto();
void skip_edline();
void lastline();
void newline();
void display_footnote();
void run_command();
void clear_input();
void redraw_input();
void clear_prompt();
void redraw_prompt();
void clear_all();
void redraw_all();
void overtype_char();
void insertchars();
void deletechars();
void attr_string();
char *wrap_out();
int pager();
int check_winsize();

/* macros to find cursor position from input buffer position */
/* NOTE: use unsigned division perhaps? */
#define CURLINE(pos) (((pos) + col0) / cols_1 + line0)
#define CURCOL(pos) (((pos) + col0) % cols_1)

/* NOTE: anyone knows if ANSI 6429 talks about more than 8 colours? */
typedef enum {
    ATTR_BLACK, ATTR_RED, ATTR_GREEN, ATTR_YELLOW,
    ATTR_BLUE, ATTR_MAGENTA, ATTR_CYAN, ATTR_WHITE,
    ATTR_FILL, ATTR_COLOURS = ATTR_FILL, ATTR_NONE = ATTR_FILL,
    ATTR_BOLD, ATTR_UNDERLINE, ATTR_INVERSE, ATTR_REVERSE,
    ATTR_ERROR
} attr_token;

#define ATTR_NAMES {				\
    "black", "red", "green", "yellow",		\
    "blue", "magenta", "cyan", "white",		\
    "on",					\
    "bold", "underline", "inverse", "reverse",	\
    (char *)0					\
}

/* constructors / accessors for attribute codes */
#define ATTRCODE(fg, bg) ((fg) | ((bg) << 8))
#define FOREGROUND(attrcode) ((attr_token)((attrcode) & 0xff))
#define BACKGROUND(attrcode) ((attr_token)((attrcode) >> 8))

#define NOATTRCODE ATTRCODE((attr_token)ATTR_NONE, (attr_token)ATTR_NONE)

/*
 * NCSA telnet 2.2 doesn't reset the colour when it receives "esc [ m",
 * so we must know what the normal colours are in order to reset it.
 * These colours can be changed with the #stdcolour command.
 */
#ifdef TELNETBUG
#define DEFAULTFG ((attr_token)ATTR_WHITE)
#define DEFAULTBG ((attr_token)ATTR_BLUE)
#endif

